
#include "bootloader.h"

FATFS	fs;

BYTE	membuff[SPM_PAGESIZE];
BYTE	binbuff[MAXBIN];
BYTE	readbuf[READBUFNUM];
UINT	buffptr, br;

void boot_program_page (DWORD page, BYTE *buf)
{
	UINT	i;
	WORD	w;

	eeprom_busy_wait();
	boot_page_erase(page);
	boot_spm_busy_wait();

	for (i = 0;i < SPM_PAGESIZE; i += 2) {
		w = *buf++;
		w += (*buf++) << 8;
		boot_page_fill(page + i, w);
	}

	boot_page_write(page);
	boot_spm_busy_wait();
	boot_rww_enable();
}

FRESULT readdata(BYTE *rbuff)
{
	FRESULT	res;

	buffptr ++;

	if (buffptr < br) {
		*rbuff = readbuf[buffptr];
		return 0;
	}
	if (br < READBUFNUM) {
//		return FR_DISK_ERR;
		return 8;
	}

	buffptr = 0;

	res = pf_read(readbuf, READBUFNUM, &br);
	*rbuff = readbuf[buffptr];

	return res;

}

FRESULT bootloader(void)
{
	BYTE	fld = 0;
	BYTE	ptr = 0;
	BYTE	dtptr = 0;
	BYTE	stflag = 1;

	BYTE	rbuff, rdt;
	BYTE	rtype = 0;
	BYTE	dt = 0;
	BYTE	bc = 0;
	WORD	add = 0;
	DWORD	offadd = 0;
	DWORD	i, k, page, wrpage = 0;

	FRESULT	res;

	if ( IS_SD_EJECT ) {
		return 0;
	}

	for (i = 0;;i ++) {
		res = pf_mount(&fs);

		if (!res) {
			break;
		}
		if (i >= 50) {
			return 0;
		}
		_delay_ms(100);
	}

	res = pf_open(AVR_HEXFILE);
	if ( res ) {
		return 0;
	}

	buffptr = READBUFNUM;
	br = buffptr;

	for (i = 0;i < SPM_PAGESIZE;i ++) {
		membuff[i] = 0xff;
	}

	fld = 0;
	ptr = 0;
	dtptr = 0;
	for (;;) {

		if ( IS_SD_EJECT ) {
			return FR_NO_FILESYSTEM;
		}

		res = readdata(&rbuff);
		if ( res ) {
			return res;
		}

		rdt = (rbuff >= 'A') ? (rbuff-'A'+10) : (rbuff-'0');

		switch (fld) {
			case 0:						// oCgJEgT
				LEDR_ON();
				if ( rbuff == ':' ) {
					ptr = 0;
					fld = 1;
				}
				break;
			case 1:						// oCgJEg
				if (ptr == 0) {
					bc = rdt * 0x10;
					ptr ++;
				} else {
					bc += rdt;
					ptr = 0;
					fld = 2;
				}
				break;
			case 2:						// AhX
				if (ptr == 0) {
					add = rdt * 0x1000;
					ptr ++;
				} else if (ptr == 1) {
					add += (rdt * 0x100);
					ptr ++;
				} else if (ptr == 2) {
					add += (rdt * 0x10);
					ptr ++;
				} else {
					add += rdt;
					ptr = 0;
					fld = 3;
				}
				break;
			case 3:						// R[h^Cv
				if (ptr == 0) {
					rtype = rdt * 0x10;
					ptr ++;
				} else {
					rtype += rdt;
					ptr = 0;
					fld = 4;
					dtptr = 0;
				}
				break;
			case 4:						// f[^
				LEDR_OFF();
				if (ptr == 0) {
					dt = rdt * 0x10;
					ptr ++;
				} else {
					dt += rdt;
					binbuff[dtptr] = dt;
					dtptr ++;
					ptr = 0;
					if (dtptr >= bc) {
						fld = 5;
					}
				}
				break;
			case 5:
				fld = 0;
				break;
		}

		if (fld == 5) {
			if (rtype == 0x02) {			// gZOg

				offadd = ((DWORD)binbuff[0]<<12) + ((DWORD)binbuff[1]<<4);

			} else if (rtype == 0x00) {		// f[^

				for (k = 0;k < bc;k ++) {

					page = ( (offadd+(DWORD)(add+k)) & ~(SPM_PAGESIZE-1) );
					if ( stflag ) {
						wrpage = page;
						stflag = 0;
					}

					if (wrpage != page) {
						boot_program_page(wrpage, membuff);
						wrpage = page;
						for (i = 0;i < SPM_PAGESIZE;i ++) {
							membuff[i] = 0xff;
						}
					}

					membuff[ (add+k) & (SPM_PAGESIZE-1) ] = binbuff[k];
				}

			} else if (rtype == 0x01) {		// I

				boot_program_page(wrpage, membuff);
				LEDR_OFF();
				return 255;

			} else {						// T|[gO
				return 10;
			}
		}
	}

	return 0;
}
