
#include "bootloader.h"

#define	MAXLEN		128

extern FATFS	fs;
extern BYTE		binbuff[MAXBIN];
extern UINT		buffptr, br;

char	tdipat[MAXLEN];
char	tdopat[MAXLEN];
char	maskpat[MAXLEN];
char	tmppat[MAXLEN];

BYTE change_state[TAP_STATE_NUM][TAP_STATE_NUM] = {
//	  U   I  SD  SI  PI
	{99,  1, 99, 99, 99},	// UNKNOWN  -> next
	{99,  0,  2,  3, 99},	// RUN_IDLE -> next
	{99,  4, 99, 99, 99},	// SHIFT_DR -> next
	{99, 99, 99, 99,  5},	// SHIFT_IR -> next
	{99,  6,  7,  8, 99}	// PAUSE_IR -> next
};

FRESULT svfloader(void)
{
	char	*bufptr;
	DWORD	len;
	BYTE	rbuff;
	BYTE	lineptr;
	UINT	i;

	FRESULT	res;

	TAP_STATE	now_state;

	if ( IS_SD_EJECT ) {
		return 0;
	}

	for (i = 0;;i ++) {
		res = pf_mount(&fs);

		if (!res) {
			break;
		}
		if (i >= 50) {
			return 0;
		}
		_delay_ms(100);
	}

	res = pf_open(AVR_SVFFILE);
	if ( res ) {
		return 0;
	}

	buffptr = READBUFNUM;
	br = buffptr;

	now_state = UNKNOWN;
	lineptr = 0;
	strcpy(tdipat,  "(FFFF)");
	strcpy(tdopat,  "(FFFF)");
	strcpy(maskpat, "(FFFF)");

	DDRC  = 0b01110000;		/* TDO, TCK=1, TMS=1, TDI=1, -, -, SDA, SCL */
	PORTC = 0b11111111;		/* vAbv */

	for (;;) {

		if ( IS_SD_EJECT ) {
			return FR_NO_FILESYSTEM;
		}

		res = readdata(&rbuff);
		if ( res == 8 ) {		// file end
			break;
		}
		if ( res ) {
			return res;
		}

		LEDR_ON();
		binbuff[lineptr] = rbuff;
		lineptr ++;
		if (rbuff != 0x0a) {
			continue;
		}

		lineptr --;
		lineptr --;
		binbuff[lineptr] = 0;
		lineptr = 0;
		LEDR_OFF();

// detect comment

		if (strncmp((char*)binbuff, "!", 1) == 0) {
			// no action
			continue;
		}
		if (strncmp((char*)binbuff, "//", 2) == 0) {
			// no action
			continue;
		}

// detect other command
		if (strncmp((char*)binbuff, "FREQUENCY ", 10) == 0) {
			// no action
			continue;
		}
		if (strncmp((char*)binbuff, "TRST ", 5) == 0) {
			// no action
			continue;
		}
		if (strncmp((char*)binbuff, "ENDDR IDLE", 10) == 0) {
			// no action
			continue;
		}
		if (strncmp((char*)binbuff, "ENDIR IRPAUSE", 13) == 0) {
			// no action
			continue;
		}
		if ( strncmp((char*)binbuff, "STATE IDLE", 10) == 0) {
			state_out(&now_state, RUN_IDLE);
			continue;
		}


// detect main command
		bufptr = strtok((char*)binbuff, " ;");

// RUNTEST
		if (strcmp(bufptr, "RUNTEST") == 0) {			// 1
			bufptr = strtok(NULL, " ;");
			if ( strcmp(bufptr, "IDLE") == 0) {			// 2
				bufptr = strtok(NULL, " ;");
			}
			state_out(&now_state, RUN_IDLE);
			len = atol(bufptr);							// 2
			bufptr = strtok(NULL, " ;");				// 3
			if (strcmp(bufptr, "TCK") != 0) {
				return 10;								// error
			}
			bufptr = strtok(NULL, " ;");				// 4
			if (bufptr != NULL) {
				if (strcmp(bufptr, "ENDSTATE") != 0) {
					return 10;							// error
				}
				bufptr = strtok(NULL, " ;");
				if ( strcmp(bufptr, "IDLE") != 0) {		// 5
					return 10;							// error
				}
			}
			set_tcksig(len);
			state_out(&now_state, RUN_IDLE);
			continue;
		}

// SIR
		if (strcmp(bufptr, "SIR") == 0) {				// 1
			len = atol( strtok(NULL, " ;") );			// 2
			bufptr = strtok(NULL, " ;");				// 3
			if (strcmp(bufptr, "TDI") != 0) {
				return 10;								// error
			}
			strcpy(tdipat, strtok(NULL, " ;") );		// 4
			bufptr = strtok(NULL, " ;");				// 5
			if (bufptr != NULL) {
				return 10;								// error
			}
			state_out(&now_state, SHIFT_IR);
			set_tdisig(len, tdipat);
			state_out(&now_state, PAUSE_IR);
			continue;
		}

// SDR
		if (strcmp(bufptr, "SDR") == 0) {				// 1
			len = atol(strtok(NULL, " ;") );			// 2
			bufptr = strtok(NULL, " ;");				// 3
			if (strcmp(bufptr, "TDI") != 0) {
				return 10;								// error
			}
			strcpy(tdipat, strtok(NULL, " ;") );		// 4
			bufptr = strtok(NULL, " ;");				// 5
			if (bufptr == NULL) {
				state_out(&now_state, SHIFT_DR);
				set_tdisig(len, tdipat);
				state_out(&now_state, RUN_IDLE);
				continue;
			}
			if (strcmp(bufptr, "TDO") != 0) {
				return 10;								// error
			}
			strcpy(tdopat, strtok(NULL, " ;") );		// 6
			bufptr = strtok(NULL, " ;");				// 7
			if (bufptr == NULL) {
				state_out(&now_state, SHIFT_DR);
				res = set_tdiosig(len, tdipat, tdopat, maskpat);
				if (res) {
					return res;
				}
				state_out(&now_state, RUN_IDLE);
				continue;
			}
			if (strcmp(bufptr, "MASK") != 0) {
				return 10;								// error
			}
			strcpy(maskpat, strtok(NULL, " ;") );		// 8
			bufptr = strtok(NULL, " ;");				// 9
			if (bufptr != NULL) {
				return 10;								// error
			}
			state_out(&now_state, SHIFT_DR);
			res = set_tdiosig(len, tdipat, tdopat, maskpat);
			if (res) {
				return res;
			}
			state_out(&now_state, RUN_IDLE);
			continue;
		}


// command error

		return 10;			// error
	}

	LEDR_OFF();
	return 254;				// normal end
}

void state_out(TAP_STATE *now_state, TAP_STATE next_state)
{
	unsigned char ns;

	ns = change_state[*now_state][next_state];
	switch (ns) {
		case 0:			// ړȂ
			break;
		case 1:			// UNKNOWN -> IDLE
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(0,0);
			break;
		case 2:			// IDLE -> SHIFT_DR
			set_tapsig(1,0);
			set_tapsig(0,0);
			set_tapsig(0,0);
			break;
		case 3:			// IDLE -> SHIFT_IR
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(0,0);
			set_tapsig(0,0);
			break;
		case 4:			// SHIFT_DR(EXIT1_DR) -> IDLE
			set_tapsig(1,0);
			set_tapsig(0,0);
			break;
		case 5:			// SHIFT_IR(EXIT1_IR) -> PAUSE_IR
			set_tapsig(0,0);
			break;
		case 6:			// PAUSE_IR -> IDLE
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(0,0);
			break;
		case 7:			// PAUSE_IR -> SHIFT_DR
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(1,0);
			set_tapsig(0,0);
			set_tapsig(0,0);
			break;
		case 8:			// PAUSE_IR -> SHIFT_IR
			set_tapsig(1,0);
			set_tapsig(0,0);
			break;
		default:		// G[
//			fprintf(stderr, "STATE Error %d->%d\n", *now_state, next_state);
			break;
	}
	*now_state = next_state;
}

BYTE set_tapsig(BYTE tms, BYTE tdi)
{
	BYTE	port_tap;
	BYTE	port_out;
	BYTE	tdo;

	port_out = (tms << 5) + (tdi << 4);

	port_tap  = PORTC;
	port_tap &= ~0b01110000;	/* TCK, TMS, TDI */
	port_tap |=  port_out;		/* TCK, TMS, TDI */
	PORTC = port_tap;			/* TCK=0 */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	PORTC |= 0b01000000;		/* TCK=1 */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	nop();						/* ԑ҂ */
	tdo = (PINC & 0b10000000) ? 1 : 0;
	return tdo;
}

void set_tcksig(DWORD len)
{
	DWORD	i;
	BYTE	port_out0;
	BYTE	port_out1;

	port_out0 = PORTC;
	port_out0 &= ~0b01110000;	/* TCK=0, TMS=0, TDI=0 */
	port_out1 = port_out0;
	port_out1 |=  0b01000000;	/* TCK=1, TMS=0, TDI=0 */

	for (i = 0;i < len;i ++) {
 		PORTC = port_out0;			/* TCK=0 */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
 		PORTC = port_out1;			/* TCK=1 */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
		nop();						/* ԑ҂ */
	}

	return;
}

void conv_hex2bin(char *srcpat)
{
	int		i,j;

	strcpy(tmppat, srcpat);

	for (i = 0;i < MAXLEN;i ++) {
		srcpat[i] = 0;
	}

	j = 0;

	for (i = strlen(tmppat)-1;i >= 0;i --) {
		switch (tmppat[i]) {
			case '(':
			case ')':
				break;
			case '0': srcpat[j+3]=0; srcpat[j+2]=0; srcpat[j+1]=0; srcpat[j+0]=0; j+=4; break;
			case '1': srcpat[j+3]=0; srcpat[j+2]=0; srcpat[j+1]=0; srcpat[j+0]=1; j+=4; break;
			case '2': srcpat[j+3]=0; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case '3': srcpat[j+3]=0; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			case '4': srcpat[j+3]=0; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=0; j+=4; break;
			case '5': srcpat[j+3]=0; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=1; j+=4; break;
			case '6': srcpat[j+3]=0; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case '7': srcpat[j+3]=0; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			case '8': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=0; srcpat[j+0]=0; j+=4; break;
			case '9': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=0; srcpat[j+0]=1; j+=4; break;
			case 'A': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case 'B': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			case 'C': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=0; j+=4; break;
			case 'D': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=1; j+=4; break;
			case 'E': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case 'F': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			case 'a': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case 'b': srcpat[j+3]=1; srcpat[j+2]=0; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			case 'c': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=0; j+=4; break;
			case 'd': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=0; srcpat[j+0]=1; j+=4; break;
			case 'e': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=0; j+=4; break;
			case 'f': srcpat[j+3]=1; srcpat[j+2]=1; srcpat[j+1]=1; srcpat[j+0]=1; j+=4; break;
			default:
				break;
		}
	}
}

void set_tdisig(DWORD len, char *tdipat)
{
	DWORD	i;
	BYTE	tms;

	conv_hex2bin(tdipat);

	for (i = 0;i < len;i ++) {
		tms = (i == len-1) ? 1 : 0;
		set_tapsig(tms,tdipat[i]);
	}
}

FRESULT set_tdiosig(DWORD len, char *tdipat, char *tdopat, char *maskpat)
{
	DWORD	i;
	BYTE	tdo, tms;

	conv_hex2bin(tdipat);
	conv_hex2bin(tdopat);
	conv_hex2bin(maskpat);

	for (i = 0;i < len;i ++) {
		tms = (i == len-1) ? 1 : 0;
		tdo = set_tapsig(tms,tdipat[i]);
		if (maskpat[i] == 1) {
			if (tdopat[i] != tdo) {
				return 12;
			}
		}
	}

	return 0;
}

