--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity AVRIF is
	port (
		ADD			: in  std_logic_vector(2 downto 0);
		DTI			: in  std_logic_vector(7 downto 0);
		PAI			: in  std_logic_vector(7 downto 0);
		PBI			: in  std_logic_vector(7 downto 0);
		PCI			: in  std_logic_vector(3 downto 0);
		WRMODEN		: in  std_logic;
		DTOUTN		: in  std_logic;
		DTWRN		: in  std_logic;
		A_WRN		: in  std_logic;
		RSTN		: in  std_logic;
		PAO			: out std_logic_vector(7 downto 0);
		PBO			: out std_logic_vector(7 downto 0);
		PCO			: out std_logic_vector(3 downto 0);
		DTO			: out std_logic_vector(7 downto 0);
		AVRRSTN		: out std_logic;
		ROMSEL		: out std_logic;
		ROMAD		: out std_logic_vector(18 downto 0);
		ROMDT		: out std_logic_vector(7 downto 0);
		ROMWRN		: out std_logic;
		ROMWRMODEN	: out std_logic;
		DTOCNTN		: out std_logic;
		P6_NRSTOUT	: out std_logic
	);
end AVRIF;

architecture RTL of AVRIF is

	signal port_ao		: std_logic_vector(7 downto 0);
	signal port_bo		: std_logic_vector(7 downto 0);
	signal port_co		: std_logic_vector(3 downto 0);

	signal do_i			: std_logic_vector(7 downto 0);
	signal romsel_i		: std_logic;
	signal romwrcnt_i	: std_logic_vector(18 downto 0);

	signal avrrstn_i	: std_logic;

begin

-- PIO(8255)
-- data input
	process (A_WRN,RSTN)
	begin
		if (RSTN = '0') then
			port_ao <= (others => '0');
			port_bo <= (others => '0');
			port_co <= (others => '0');
		elsif (A_WRN'event and A_WRN = '1') then
			if    (ADD = "000") then
				port_ao <= DTI;
			elsif (ADD = "001") then
				port_bo <= DTI;
			elsif (ADD = "010") then
				port_co <= DTI(7 downto 4);
			end if;
		end if;
	end process;

	PAO <= port_ao;
	PBO <= port_bo;
	PCO <= port_co;

-- data output
	do_i <=
		PAI				when (ADD = "000") else
		PBI				when (ADD = "001") else
		"0000" & PCI;

	DTO <= do_i;


-- AVR reset control
	process (A_WRN)
	begin
		if (A_WRN'event and A_WRN = '1') then
			if (ADD = "111") then
				avrrstn_i <= DTI(0);
			end if;
		end if;
	end process;

	AVRRSTN <= avrrstn_i;


-- ROM bank setting
	process (A_WRN,avrrstn_i)
	begin
		if (avrrstn_i = '0') then
			romsel_i <= '0';
		elsif (A_WRN'event and A_WRN = '1') then
			if (ADD = "011") then
				romsel_i <= DTI(0);
			end if;
		end if;
	end process;

	ROMSEL <= romsel_i;


-- ROM write control
	process (A_WRN)
	begin
		if (A_WRN'event and A_WRN = '1') then
			if    (ADD = "100") then
				romwrcnt_i <= (others => '0');
			elsif (ADD = "101") then
				romwrcnt_i <= romwrcnt_i + 1;
			end if;
		end if;
	end process;

	ROMAD   <= romwrcnt_i;
	ROMDT   <= DTI;

	ROMWRMODEN <= WRMODEN;
	ROMWRN     <= DTWRN;
	DTOCNTN    <= DTOUTN;
	P6_NRSTOUT <= RSTN;

end RTL;
