--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity P6IF is
	port (
		ADD			: in  std_logic_vector(15 downto 0);
		DTI			: in  std_logic_vector(7 downto 0);
		IORQN		: in  std_logic;
		WRN			: in  std_logic;
		RDN			: in  std_logic;
		MEMCSN		: in  std_logic;
		PIOCSN		: in  std_logic;
		ROMSEL		: in  std_logic;
		PAI			: in  std_logic_vector(7 downto 0);
		PBI			: in  std_logic_vector(7 downto 0);
		PCI			: in  std_logic_vector(3 downto 0);
		CLK			: in  std_logic;
		AVRRSTN		: in  std_logic;
		RSTN		: in  std_logic;
		BANK0		: out std_logic_vector(5 downto 0);
		BANK1		: out std_logic_vector(5 downto 0);
		BANK2		: out std_logic_vector(5 downto 0);
		BANK3		: out std_logic_vector(5 downto 0);
		BANK4		: out std_logic_vector(5 downto 0);
		BANK5		: out std_logic_vector(5 downto 0);
		BANK6		: out std_logic_vector(5 downto 0);
		BANK7		: out std_logic_vector(5 downto 0);
		ROMSET		: out std_logic_vector(1 downto 0);
		RAMSET		: out std_logic_vector(1 downto 0);
		CS01R		: out std_logic;
		CS01RX		: out std_logic;
		CS01W		: out std_logic;
		CS02W		: out std_logic;
		PC60MODE	: out std_logic;
		ROMXMODE	: out std_logic;
		CSCG		: out std_logic;
		DOBANK		: out std_logic_vector(7 downto 0);
		DO			: out std_logic_vector(7 downto 0);
		PAO			: out std_logic_vector(7 downto 0);
		PBO			: out std_logic_vector(7 downto 0);
		PCO			: out std_logic_vector(3 downto 0)
	);
end P6IF;

architecture RTL of P6IF is

	type Bank_type is array (0 to 7) of std_logic_vector(5 downto 0);
	signal bankset_i	: Bank_type;
	signal romset_i		: std_logic_vector(1 downto 0);
	signal ramset_i		: std_logic_vector(1 downto 0);

	signal rstbk0		: std_logic_vector(5 downto 0);
	signal rstbk1		: std_logic_vector(5 downto 0);
	signal rstbk2		: std_logic_vector(5 downto 0);
	signal rstbk3		: std_logic_vector(5 downto 0);
	signal rstbk4		: std_logic_vector(5 downto 0);
	signal rstbk5		: std_logic_vector(5 downto 0);
	signal rstbk6		: std_logic_vector(5 downto 0);
	signal rstbk7		: std_logic_vector(5 downto 0);

	signal kyusol		: std_logic;
	signal ram16k		: std_logic;

	signal cs01r_i		: std_logic;
	signal cs01rx_i		: std_logic;
	signal cs01w_i		: std_logic;
	signal cs02w_i		: std_logic;
	signal pc60_i		: std_logic;
	signal romx_i		: std_logic;
	signal cscg_i		: std_logic;

	signal bank_do		: std_logic_vector(5 downto 0);
	signal dobank_i		: std_logic_vector(7 downto 0);

	signal port_ao		: std_logic_vector(7 downto 0);
	signal port_bo		: std_logic_vector(7 downto 0);
	signal port_co		: std_logic_vector(3 downto 0);

	signal do_i			: std_logic_vector(7 downto 0);

	signal a_o			: std_logic;
	signal b_o			: std_logic;

begin

	rstbk0 <=
		"010000" when (ROMSEL = '0') else	-- 64KRAM
		"111111" when (ram16k = '1') else	-- 16KRAM
		"111111" when (kyusol = '1') else	-- m
		"010000";							-- ʏROM
	rstbk1 <=
		"010001" when (ROMSEL = '0') else
		"111111" when (ram16k = '1') else
		"111111" when (kyusol = '1') else
		"010001";
	rstbk2 <=
		"010010" when (ROMSEL = '0') else
		"000000" when (ram16k = '1') else
		"000000" when (kyusol = '1') else
		"000000";
	rstbk3 <=
		"010011" when (ROMSEL = '0') else
		"000001" when (ram16k = '1') else
		"010011" when (kyusol = '1') else
		"000001";
	rstbk4 <=
		"010100" when (ROMSEL = '0') else
		"111111" when (ram16k = '1') else
		"010100" when (kyusol = '1') else
		"010100";
	rstbk5 <=
		"010101" when (ROMSEL = '0') else
		"111111" when (ram16k = '1') else
		"010101" when (kyusol = '1') else
		"010101";
	rstbk6 <=
		"010110" when (ROMSEL = '0') else
		"111111" when (ram16k = '1') else
		"111111" when (kyusol = '1') else
		"010110";
	rstbk7 <=
		"010111" when (ROMSEL = '0') else
		"111111" when (ram16k = '1') else
		"111111" when (kyusol = '1') else
		"010111";

-- Bank Register
	process (RSTN,CLK,rstbk0,rstbk1,rstbk2,rstbk3,rstbk4,rstbk5,rstbk6,rstbk7)
	begin
		if (RSTN = '0') then
			bankset_i(0) <= rstbk0;		-- AreaA (0000-1FFFH)
			bankset_i(1) <= rstbk1;		-- AreaB (2000-3FFFH)
			bankset_i(2) <= rstbk2;		-- AreaC (4000-5FFFH)
			bankset_i(3) <= rstbk3;		-- AreaD (6000-7FFFH)
			bankset_i(4) <= rstbk4;		-- AreaE (8000-9FFFH)
			bankset_i(5) <= rstbk5;		-- AreaF (A000-BFFFH)
			bankset_i(6) <= rstbk6;		-- AreaG (C000-DFFFH)
			bankset_i(7) <= rstbk7;		-- AreaH (E000-FFFFH)
			romset_i     <= "00";
			ramset_i     <= "00";
		elsif (CLK'event and CLK = '0') then
			if (IORQN = '0' and WRN = '0') then
				if (MEMCSN = '0') then						-- 0x30-0x3f
					if    (DTI(7 downto 6) = "00") then
						romset_i <= DTI(5 downto 4);
					elsif (DTI(7 downto 6) = "01") then
						ramset_i <= DTI(5 downto 4);
					end if;
					bankset_i( conv_integer(ADD(2 downto 0) ) ) <= DTI(7 downto 6) & DTI(3 downto 0);
				elsif (ADD(7 downto 4) = X"7") then		-- 0x70-0x7f
					bankset_i(2) <= "00" & DTI(3 downto 0); 
				end if;
			end if;
		end if;
	end process;

-- Bank Register
	process (AVRRSTN,CLK)
	begin
		if (AVRRSTN = '0') then
			kyusol <= '0';
			ram16k <= '0';
		elsif (CLK'event and CLK = '0') then
			if (IORQN = '0' and WRN = '0') then
				if (MEMCSN = '0') then						-- 0x30-0x3f
					if    (ADD(2 downto 0) = "011") then
						if (DTI = X"43") then
							kyusol <= '1';
						else
							kyusol <= '0';
						end if;
					elsif (ADD(2 downto 0) = "100") then
						if (DTI(7 downto 6) = "11") then
							ram16k <= '1';
						else
							ram16k <= '0';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;

	BANK0  <= bankset_i(0);
	BANK1  <= bankset_i(1);
	BANK2  <= bankset_i(2);
	BANK3  <= bankset_i(3);
	BANK4  <= bankset_i(4);
	BANK5  <= bankset_i(5);
	BANK6  <= bankset_i(6);
	BANK7  <= bankset_i(7);
	ROMSET <= romset_i;
	RAMSET <= ramset_i;

	bank_do <= bankset_i( conv_integer( ADD(2 downto 0) ) );

	dobank_i(7 downto 6) <= bank_do(5 downto 4);
	dobank_i(3 downto 0) <= bank_do(3 downto 0);
	dobank_i(5 downto 4) <= 
		romset_i when (bank_do(5 downto 4) = "00") else
		ramset_i when (bank_do(5 downto 4) = "01") else
		"11";

	DOBANK <= dobank_i;


-- I/O Port Register
	process (RSTN,CLK)
	begin
		if (RSTN = '0') then
			cs01r_i  <= '0';
			cs01rx_i <= '0';
			cs01w_i  <= '0';
			cs02w_i  <= '0';
			pc60_i   <= '0';
			romx_i   <= '0';
			cscg_i   <= '0';
		elsif (CLK'event and CLK = '0') then
			if (IORQN = '0' and WRN = '0') then
				if (ADD(7 downto 0) = X"06") then		-- 0x06
					if (DTI = X"66") then
						pc60_i <= '1';
					else
						pc60_i <= '0';
					end if;
					if (DTI = X"77") then
						romx_i <= '1';
					else
						romx_i <= '0';
					end if;
				end if;
				if (ADD(7 downto 0) = X"F0") then		-- 0xF0
					if (
						DTI(3 downto 0) = X"0" or
						DTI(3 downto 0) = X"1" or
						DTI(3 downto 0) = X"2" or
						DTI(3 downto 0) = X"5" or
						DTI(3 downto 0) = X"6" or
						DTI(3 downto 0) = X"F"
					) then
						cs01r_i <= '0';
					else
						cs01r_i <= '1';
					end if;
					if (DTI(3 downto 0) = X"1") then
						cs01rx_i <= '1';
					else
						cs01rx_i <= '0';
					end if;
				end if;
				if (ADD(7 downto 0) = X"F2") then		-- 0xF2
					if (DTI(1 downto 0) = "00") then
						cs01w_i <= '0';
					else
						cs01w_i <= '1';
					end if;
					if (DTI(3 downto 2) = "00") then
						cs02w_i <= '0';
					else
						cs02w_i <= '1';
					end if;
				end if;
				if (ADD(7 downto 0) = X"92") then		-- 0x92
					if    (DTI(2) = '0') then
						cscg_i <= '1';
					elsif (DTI(2) = '1') then
						cscg_i <= '0';
					end if;
				elsif (ADD(7 downto 0) = X"93") then	-- 0x93
					if    (DTI = X"04") then
						cscg_i <= '1';
					elsif (DTI = X"05") then
						cscg_i <= '0';
					end if;
				end if;
			end if;
		end if;
	end process;

	CS01R	 <= cs01r_i;
	CS01RX	 <= cs01rx_i;
	CS01W	 <= cs01w_i;
	CS02W	 <= cs02w_i;
	PC60MODE <= pc60_i;
	ROMXMODE <= romx_i;
	CSCG	 <= cscg_i;


-- PIO(8255)
-- data input
	process (CLK,RSTN)
	begin
		if (RSTN = '0') then
			port_ao <= (others => '0');
			port_bo <= (others => '0');
			port_co <= (others => '0');
			a_o <= '0';
			b_o <= '0';
		elsif (CLK'event and CLK = '1') then
			if (IORQN = '0' and WRN = '0' and PIOCSN = '0') then			-- 0xd0-0xdf / 0x10-0x1f
				if    (ADD(3 downto 0) = "0000") then
					port_ao <= DTI;
				elsif (ADD(3 downto 0) = "0001") then
					port_bo <= DTI;
				elsif (ADD(3 downto 0) = "0010") then
					port_co <= DTI(7 downto 4);
				elsif (ADD(3 downto 0) = "0011") then
					case DTI is
						when X"91"  => a_o <= '0'; b_o <= '1';	-- A:in  B:out CH:out CL:in
						when X"93"  => a_o <= '0'; b_o <= '0';	-- A:in  B:in  CH:out CL:in
						when X"81"  => a_o <= '1'; b_o <= '1';	-- A:out B:out CH:out CL:in
						when X"08"  => port_co(0) <= '0';
						when X"09"  => port_co(0) <= '1';
						when X"0A"  => port_co(1) <= '0';
						when X"0B"  => port_co(1) <= '1';
						when X"0C"  => port_co(2) <= '0';
						when X"0D"  => port_co(2) <= '1';
						when X"0E"  => port_co(3) <= '0';
						when X"0F"  => port_co(3) <= '1';
						when others => null;
					end case;
				end if;
			end if;
		end if;
	end process;

	PAO <= port_ao;
	PBO <= port_bo;
	PCO <= port_co;


-- data output
	do_i <=
		PAI     when (ADD(1 downto 0) = "00" and a_o = '0') else
		port_ao when (ADD(1 downto 0) = "00" and a_o = '1') else
		X"00"   when (ADD(1 downto 0) = "00" and a_o = '1') else
		PBI     when (ADD(1 downto 0) = "01" and b_o = '0') else
		port_bo when (ADD(1 downto 0) = "01" and b_o = '1') else
		X"00"   when (ADD(1 downto 0) = "01" and b_o = '1') else
		port_co & PCI;

	DO <= do_i;



end RTL;
